/*****************************************************
File    			: AkkuTester
Version 			: v. 1.3
Create  			: 26.07.2008
Modify  			: 11.05.2008
Author  			: Philipp Klin
-----------------------------------------------------
Chip type           : ATmega32
Clock frequency     : 8 MHz (Int Oscillator)
Memory model        : mega
Flash Memory size   : 32'768 Byte
SRAM Memory size    : 2'048 Byte
*****************************************************/

//***********
//* Includes
//***********
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "global.h"
#include "init.h"
#include "glcd.h"
#include "application.h"
#include "clocktime.h"
#include "eeprom.h"
#include "dos.h"
#include "lang.h"


//************
//* Variabeln
//************
volatile unsigned char name;
volatile unsigned char i;


/**
 * \brief
 * Hauptprogramm
*/
int main(void)
{
	//******************
	//* Initialisierung
	//******************

	// LCD initialisieren
	ks0108Init(FALSE);

	// Encoder initialisieren
	EncoderInit();

	// Taste initialisieren	
	TasteInit();

	// Initialisiert die Pins der AD-Eingnge
	ADPinsInit();

	// Initialisiert die PWM Ausgangspins
	PWMPinsInit();

	// Initialisiert den AD Konverter
	ADC_Init();

	// Initialisiert den Timer2 fr die 1ms Zeitbasis
	Timer2_Init();

	// Initialisiert den Timer1 fr die PWM Ausgnge
	Timer1_Init();

	// SplashScreen
	SplashScreen();

	// Alle Interrupts enable
	sei();

	// Einstellungen aus EEPROM laden
	EEPROM_To_SRAM();

	//****************
	//* Hauptprogramm
	//****************
	Application();
}


/**
* \brief
* Timer0 Compare Match Interrupt <br>
* wird zur Erzeugung der Zeit benutzt
*/
ISR (TIMER2_COMP_vect)
{
	MSecTick();
}
  
