/**************
 application.c
***************/


//***********
//* Includes
//***********
#include "application.h"


/**
* \brief
* Diese Funktion fuehrt die Hauptanwendugn aus, die Hauptsaechlich aus der Bedienung besteht
* \version
* 1.1 Letzter Eintrag wird gemerkt
* 1.0 Erste Version
*/
void Application ( void ) {
	AnswMenue_Betriebswahl answer;

	OKEntprellen();
	TasteOK.Kurz = FALSE;
	
	while (1) {
		answer = DrawMenue(mnuBetriebswahl, 0, FALSE);
		OKEntprellen();
		TasteOK.Kurz = FALSE;
	
		switch (answer) {
			case answAkkuEntladen:
				// Akku Entladen
				Entladen();
				break;
			
			case answRiMessen:
				// RI messen
				RiMessen();
				break;

			case answGlobaleEinstellungen:
				// Globale Einstellungen
				GlobalSettings();
				break;

			default:
				break;
		}

		OKEntprellen();
		TasteOK.Kurz = FALSE;
	}
}


//*************************************************************
/// \brief
/// Globale Einstellungen vornehmen
//*************************************************************
void GlobalSettings ( void ) {
	AnswMenue_GlobaleEinstellungen	answer;

	OKEntprellen();
	TasteOK.Kurz = FALSE;

	do {
		answer = DrawMenue(mnuGlobaleEinstellungen, 0, FALSE);
		OKEntprellen();
		TasteOK.Kurz = FALSE;

		switch (answer) {

			// Language
			case answLanguage:
				Sprache();
				break;

			// Kalibrierung
			case answKalibrierung:
				Kalibrierung();
				break;

			// Zurck
			case answZurueckGlobaleEinstellungen:
				answer = 0;
				break;

			default:
				break;
		}

		OKEntprellen();
		TasteOK.Kurz = FALSE;

	} while (answer != 0);
}


//*************************************************************
/// \brief
/// Innenwiderstand des Akku messen
//*************************************************************
void RiMessen ( void ) {
	FrameAnswer 					answSetting;
	AnswMenue_GlobaleEinstellungen	answMenu;
	unsigned char					i, j;
	unsigned char					correctValues;
	signed long					    du, di;
	unsigned long					current, voltage;
    float                           ri;


	// Einstellungen abfragen
	// **********************
	do {
		// Hauptfenster anzeigen
		OKEntprellen();
		TasteOK.Kurz = FALSE;
		answSetting = ShowSettings(2, RiSettings);
		
		if (answSetting == Aendern) {

			// Eine Einstellung ndern
			OKEntprellen();
			TasteOK.Kurz = FALSE;
			answMenu = DrawMenue(mnuEinstellungenRi, 0, FALSE);

			switch (answMenu) {

				// Entladestrom
				case answEntladestromRi:
					Entladestrom(MAX_CURRENT, &Ri.maxCurrent);
					break;

				// Messintervall
				case answMessintervallRi:
					Messintervall();
					break;

				// Zurck
				case answZurueckEinstellungenRi:
					break;

				default:
					break;
			}
		}
		
	} while (answSetting != Start);
	
	
	// Messvorgang
	// ***********

	ks0108ClearScreen();
	writeBarGraph(0, 5);

	// Einstellungen speichern
	SRAM_To_EEPROM();
	
	// Teststrom fr Testvorgang berechnen
	Ri.currentStep = Ri.maxCurrent / 20;
	for (i=0; i<21; i++) {
		Ri.currentPoints[i] = Ri.currentStep * i;
	}
	
	// Die Messungen werden nur im letzten viertel
	// des Messintervalls durchgefhrt
	Ri.startTime = (Ri.messintervall * 3000) / 4;
	
	// Zeitspanne zwischen den Messungen
	Ri.intervall = ((Ri.messintervall * 1000) - Ri.startTime) / 5;
	
	// 20 Messpunkte aufnehmen
	for (i=0; i<21; i++) {
		// Entladestrom einstellen
		DAC(Ri.currentPoints[i]);
		
		// Zeit zurckstellen
		Ri.msTick = 0;
		
		// Warten bis 75% der Zeit verstrichen
		while (Ri.msTick < Ri.startTime) {;}
		
		// 5 Spannungen und Strme einlesen
		voltage = 0;
		current = 0;
		for (j=0; j<5; j++) {
			voltage += Read_Voltage();
			current += Read_Current();
			
			Ri.msTick = 0;
			while (Ri.msTick < Ri.intervall) {;}
		}
		
		// Spannung und Strom speichern
		Ri.voltagePoints[i] = voltage / 5; 
		Ri.effectiveCurrentPoints[i] = current / 5;

		// Fortschritt anzeigen
		ks0108FillRect(0, 0, 127, 40, WHITE);
		page = 1;
		LCD_WriteInteger_Medium_Center(Ri.effectiveCurrentPoints[i], 0, 100);
		CopyLang(buffer1, txt_mA);
		column++;
		LCD_WriteString_Medium(buffer1);
		page = 3;
		LCD_WriteInteger_Medium_Center(Ri.voltagePoints[i], 0, 100);
		CopyLang(buffer1, txt_mV);
		column++;
		LCD_WriteString_Medium(buffer1);

		writeBarGraph(100/20*i, 5);
	}

	// Entladestrom einstellen
	DAC(0);

	// Innenwiderstnde ausrechnen
	Ri.maxRi 		= 0;
	Ri.middleRi 	= 0;
	correctValues 	= 0;
	for (i=0; i<20; i++) {
		du = Ri.voltagePoints[i] - Ri.voltagePoints[i+1];
		di = Ri.effectiveCurrentPoints[i+1] - Ri.effectiveCurrentPoints[i];

		// Messung auf Plausibilitt prfen
		if ( (du >= 0) && (di > 0) ) {

			// Division durch 0 Abfangen
			ri = (float)du / (float)di;
		
			// Maximaler Widerstand ausrechnen
			if (ri > Ri.maxRi) {
				Ri.maxRi = ri;
			}

			// Durchschnittlicher Widerstand ausrechnen
			Ri.middleRi += ri;
			correctValues++;
		}
	}
	Ri.middleRi /= (float)correctValues;
	
	// Ergebnis anzeigen
	// *****************
	ShowEnd(RiEnd);
	
}


/**
* \brief
* Bedienung der Entladefuntionen
* \version
* 1.1 Letzter Eintrag wird gemerkt
* 1.0 Erste Version
*/
void Entladen ( void ) {	
	unsigned char			Abbrechen;
	unsigned char 			Start			= FALSE;
	FrameAnswer 			Answer;
	unsigned char 			i;
	unsigned long 			Leistung;
	unsigned char			result;
	unsigned long 			Spannung;
	unsigned char 			NewLog 			= TRUE;
	AnswMenue_Einstellungen	AntwortES 		= 0;

	//****************
	//* Einstellungen
	//****************
	while(Start == FALSE)
	{
		// Aktuelle einstellungen zeigen		
		Answer = ShowSettings(2, DischargeSettings);

		if (Answer == Aendern)
		{
			TasteOK.Kurz = FALSE;
			
			AntwortES = DrawMenue(mnuEinstellungenEntladen, AntwortES, TRUE);
			Abbrechen = FALSE;

			switch (AntwortES)
			{
				// Akkutyp
				case answAkkutyp:
					Akkutyp();
					break;
					
				// Zellenzahl
				case answZellenzahl:
					Zellenzahl();
					break;
					
				// Entladestrom
				case answEntladestrom:
					Entladestrom(0, &Akku.Entladestrom);
					break;

				// Schlussspannung
				case answSchlussspannung:
					Schlussspannung();
					break;

				// Speicherintervall
				case answSpeicherintervall:
					Speicherintervall();
					break;

				// Dateinamen
				case answDateiname:
					LogName();
					break;

				// Werkseinstellung
				case answWerkseinstellung:
					Werkseinstellung();
					break;

				// zurck
				case answZurueckEinstellungen:
					break;
					
				// Abbrechen gedrckt
				default:
					Abbrechen = TRUE;
					break;
			}

			OKEntprellen();
			TasteOK.Kurz = FALSE;
		}
		else
		{
			// Logdateiname erstellen
			SetLogName(LogCount);

			// LogCounter frs nachste abspeichern erhhen
			LogCount++;

			// Einstellungen speichern
			SRAM_To_EEPROM();

			// Entladung starten
			Start = TRUE;
		}
	}


	//************
	//* Entladung
	//************
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// MMC/SD Hardware initialisieren
	MMC_IO_Init();

	// Header in Datei schreiben, falls mtig
	if ((Akku.Speicherintervall > 0) && (NewLog == TRUE))
	{
		// berprfen ob SD-Card eingelegt
		result = SD_CheckCard();

		// berprfen ob SD-Card eingelegt
		if (result == TRUE)
		{
			// Datei erstellen und Header schreiben
			SD_CreateLogFile();
			SD_WriteHeader();
		}
		else
		{
			NoSDCard();
			Akku.Speicherintervall = 0;
		}
	}

	// Entladestrom einstellen
	DAC(Akku.Entladestrom);

	// Ladezeit zurcksetzen
	Akku.Entladezeit = 0;
	Zeit.totSekunden = 0;

	// ShowDischargeFrame Auswahl vorbereiten
	Auswahl.MaxEintraege = 1;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = 0;
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = FALSE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Kopfzeile fr Parameter anzeigen
	ShowDischargeHeadNote();
		
	while (Start == TRUE)
	{	
		if (Auswahl.Changed == TRUE)
		{
			// Fusszeile anzeigen
			ShowDischargeFootNote();
			Auswahl.Changed = FALSE;
		}
		
		// berprfen, ob nicht berhitzt
		if (CoolerTemp > FAN_TEND)
		{
			ShowOverheat();
		}

		// berprfen, ob Taste gedrckt
		if (TasteOK.Kurz == TRUE)
		{
			Start = FALSE;
		}
		
		// berprfen, ob neue Sekunde
		if (Zeit.Changed == TRUE)
		{
			Zeit.Changed = FALSE;

			// Zeiten aktualisieren
			Akku.Entladezeit++;
			
			//**********************************
			//* Einlesen und Mittelwerte bilden
			//**********************************

			// Stromarray umkopieren und einlesen
			for (i=0; i<9; i++)
			{
				Akku.StromArray[i+1] = Akku.StromArray[i];
			}
			Akku.StromArray[0] = Read_Current();
			Akku.Strom = Read_Current();

			// Spannungsarray umkopieren und einlesen
			for (i=0; i<9; i++)
			{
				Akku.SpannungArray[i+1] = Akku.SpannungArray[i];
			}
			Akku.SpannungArray[0] = Read_Voltage();

			// Mittelwerte der Spannung ausrechnen
			for (i=0; i<10; i++)
			{
				Array[i] = Akku.SpannungArray[i];
			}
			Sortieren();
			
			Spannung = Array[3];
			Spannung += Array[4];
			Spannung += Array[5];
			Spannung += Array[6];
			Spannung /= 4;
			Akku.Spannung = (unsigned long) Spannung;

			// Mittelwerte des Stroms ausrechnen
			for (i=0; i<10; i++)
			{
				Array[i] = Akku.StromArray[i];
			}
			Sortieren();
			Akku.Strom = Array[3];
			Akku.Strom += Array[4];
			Akku.Strom += Array[5];
			Akku.Strom += Array[6];
			Akku.Strom /= 4;

			//*******************************************************
			//* Kapazitaet und Energie ausrechnen
			//*******************************************************
			/*------------------------------------
			 Kapazitt[mAh] = Strom[mA] * Zeit[s]
			 ------------------------------------*/
			Akku.Kapazitaet += Akku.Strom;

			/*-------------------------------------------------
			 Energie[mWh] = Strom[mA] * Spannung[mV] * Zeit[s]
			 --------------------------------------------------*/
			Leistung = Akku.Strom;
			Leistung *= Akku.Spannung;
			Leistung /= 1000;
			Akku.Energie += Leistung;


			//***********************************
			//* berprfen, ob Spannung zu klein
			//***********************************
			if (Abschaltung() == TRUE)
			{
				Start = FALSE;
				Auswahl.Count = 1;
			}


			//*******************************************************
			//* Logdaten speichern
			//*******************************************************
			if (Akku.Speicherintervall > 0)
			{
				if (Zeit.totSekunden > Akku.Speicherintervall)
				{
					Zeit.totSekunden = 1;
					SD_WriteLog();
				}
			}

			// Entladeeinstellungen anzeigen
			ShowDischargeContent();
		}
	}

	//******************************
	//* Entladevorgang unterbrechen
	//******************************
	// Entladestrom abschalten
	DAC(0);
	
	if (Auswahl.Count == 0)
	{	
		//*********
		//* ndern
		//*********
		NewLog = FALSE;
	}
	else
	{
		//**********
		//* Beenden
		//**********
		
		// Buffer in DAtei schreiben und abschliessen
		SD_CloseFlie();
		ks0108ClearScreen();

		// Endwerte anzeigen
		ShowEnd(EntladenEnd);

		// Variabeln neu initialisieren
		NewLog = TRUE;
		Akku.Energie = 0;
		Akku.Kapazitaet = 0;
		Akku.Spannung = 0;
		Akku.Strom = 0;
		Akku.UnterspannungCounter = 0;
		Akku.Entladezeit = 0;
		for (i=0; i<10; i++)
		{
			Akku.SpannungArray[i] = 0;
			Akku.StromArray[i] = 0;
		}
	}
}


/**
* \brief
* Ueberprueft ob abgeschaltet werden muss. <br>
* Die Abschaltung wird fuehestens 30s nach Entladebeginn aktiv
* \return
* Boolean TRUE wenn die Entladung beendet werden sollte
* \version
* 1.0 Erste Version
*/
unsigned char Abschaltung(void)
{
	unsigned char State;

	State = FALSE;	

	// berprfen, ob schon mehr als 30s entladen wurden
	if (Akku.Entladezeit > 30)
	{
		// berprfen, ob Akkuspannung zu viel unter der Ladeschlussspannung
		if (Akku.Spannung < Akku.Entladespannung)
		{
			// berprfen, ob Akku sehr viel unter der Ladeschlussspannung
			if (Akku.Spannung < (Akku.Entladespannung - MAX_UNDERVOLTAGE))
			{
				State = TRUE;
			}
		
			// Der Unterspannungscounter wird ehhh um nach einer gewissen Anzahl Sekunden abzuschalten
			Akku.UnterspannungCounter++;
			
			// berprfen, ob gengend Anzahl Sekunden bei Unterspannung um abzuschalten
			if (Akku.UnterspannungCounter > MAX_UNDERVOLTAGE_CNT)
			{
				State = TRUE;
			}
		}
		else
		{
			// Es muss nicht abgeschaltet werden
			Akku.UnterspannungCounter = 0;
		}
	}
	
	return State;
}


/**
* \brief
* Anzahl der Zellen einstellen
* \version
* 1.0 Erste Version
*/
void Zellenzahl(void)
{
	unsigned long Voltage;
	unsigned int CellVoltage;
	
	// Zellenspannung auslesen
	CellVoltage = pgm_read_word(&(CellVoltgeFull[Akku.Typ]));
	
	// Maximale Zellenzahl ausrechen
	Auswahl.MaxEintraege = EE_SCHLUSSSPANNUNG_MAX / CellVoltage;
	
	// Auswahl initialisieren
	Auswahl.MinEintraege = 1;
	Auswahl.Count = Akku.Zellenzahl;
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = TRUE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Zellenzahl);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 33, WHITE);
		
			// Zellenzah schreiben
			page = 4;
			LCD_WriteInteger_Medium_Center(Auswahl.Count, 0, 127);
			
			// Spannung schreiben
			Voltage = Auswahl.Count * CellVoltage;
			ultoa(Voltage, buffer1, 10);
			CopyLang(buffer2, txt_mV);
			strcat(buffer1, buffer2);
			page = 6;
			LCD_WriteString_Medium_Center(buffer1, 0, 127);
		}
	}

	Akku.Zellenzahl = Auswahl.Count;
	Auswahl.Enable = FALSE;

	SchlussspannungAusrechnen();
	MaxStromAusrechnen();
}


/**
* \brief
* Akkutyp einstellen
* \version
* 1.0 Erste Version
*/
void Akkutyp(void)
{
	// Auswahl initialisieren
	Auswahl.MaxEintraege = 3;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = Akku.Typ;
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = FALSE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Akkutyp);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);
		
			// Text schreiben
			page = 4;
			switch (Auswahl.Count)
			{
				// NiCd
				case 0:
					CopyLang(buffer1, txt_NiCd);
					LCD_WriteString_Medium_Center(buffer1, 0, 127);
					break;

				// NiMH
				case 1:
					CopyLang(buffer1, txt_NiMH);
					LCD_WriteString_Medium_Center(buffer1, 0, 127);
					break;

				// Blei
				case 2:
					CopyLang(buffer1, txt_Pb);
					LCD_WriteString_Medium_Center(buffer1, 0, 127);
					break;

				// Li-Ion
				case 3:
					CopyLang(buffer1, txt_LiIon);
					LCD_WriteString_Medium_Center(buffer1, 0, 127);
					break;
			} 
		}
	}

	Akku.Typ = Auswahl.Count;
	Auswahl.Enable = FALSE;

	SchlussspannungAusrechnen();
}


//*************************************************************
/// \brief
/// Entladestrom einstellen
/// \param
/// maxCurrent Maximaler Strom in mA (0: maxStrom wird berechnet)
/// \param
/// &current Pointer auf den Strom (neuer Wert wird dorthin geschrieben)
//*************************************************************
void Entladestrom ( unsigned int maxCurrent, unsigned int *currentVar )
{
	unsigned long Current;
	unsigned int CellVoltage;
	unsigned long Voltage;
	
	if (maxCurrent == 0) {
		// Zellenspannung auslesen
		CellVoltage = pgm_read_word(&(CellVoltgeFull[Akku.Typ]));
		Voltage = CellVoltage * Akku.Zellenzahl;

		// Maximalen Strom ausrechnen
		Current = MAX_POWER;
		Current /= Voltage;

		// Auf 10 genau runden
		Current /= 10;
		Current *= 10;
	} else {
		Current = maxCurrent;
	}

	// berprfen, ob Maximalstrom nicht berschritten
	if (Current > MAX_CURRENT)
	{
		Current = MAX_CURRENT;
	}

	// berprfen, ob gespeicherter Strom nicht zu gross;
	if (*currentVar > Current)
	{
		*currentVar = Current;
	}

	// Auswahl initialisieren
	Auswahl.MaxEintraege = Current;
	Auswahl.MinEintraege = 10;
	Auswahl.Count = *currentVar;
	Auswahl.Step = 10;
	Auswahl.BigStep = 100;
	Auswahl.Roll = TRUE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Entladestrom);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);
		
			// Text schreiben
			page = 4;
			ultoa(Auswahl.Count, buffer1, 10);
			CopyLang(buffer2, txt_mA);
			strcat(buffer1, buffer2);
			LCD_WriteString_Medium_Center(buffer1, 0, 127);
		}
	}

	*currentVar = Auswahl.Count;
	Auswahl.Enable = FALSE;
}


/**
* \brief
* Schlussspannung einstellen
* \version
* 1.0 Erste Version
*/
void Schlussspannung(void)
{
	// Auswahl initialisieren
	Auswahl.MaxEintraege = 30000;
	Auswahl.MinEintraege = 50;
	Auswahl.Count = Akku.Entladespannung;
	Auswahl.Step = 10;
	Auswahl.BigStep = 100;
	Auswahl.Roll = TRUE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Schlussspannung);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);
		
			// Text schreiben
			page = 4;
			ultoa(Auswahl.Count, buffer1, 10);
			CopyLang(buffer2, txt_mV);
			strcat(buffer1, buffer2);
			LCD_WriteString_Medium_Center(buffer1, 0, 127);
		}
	}

	Akku.Entladespannung = Auswahl.Count;
	Auswahl.Enable = FALSE;
}


/**
* \brief
* Speicherintervall einstellen
* \version
* 1.0 Erste Version
*/
void Speicherintervall(void)
{
	// Auswahl initialisieren
	Auswahl.MaxEintraege = 900;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = Akku.Speicherintervall;
	Auswahl.Step = 10;
	Auswahl.BigStep = 100;
	Auswahl.Roll = TRUE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Speicherintervall);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);
		
			// Text schreiben
			page = 4;
			if (Auswahl.Count != 0)
			{
				ultoa(Auswahl.Count, buffer1, 10);
				CopyLang(buffer2, txt_s);
				strcat(buffer1, buffer2);
				LCD_WriteString_Medium_Center(buffer1, 0, 127);
			}
			else
			{
				CopyLang(buffer1, txt_aus);
				LCD_WriteString_Medium_Center(buffer1, 0, 127);
			}
		}
	}

	Akku.Speicherintervall = Auswahl.Count;
	Auswahl.Enable = FALSE;
}


//*************************************************************
/// \brief
/// Messintervall fuer Ri Messen einstellen
//*************************************************************
void Messintervall ( void ) {
	// Auswahl initialisieren
	Auswahl.MaxEintraege	= MAX_MEASURE_INTERVALL;
	Auswahl.MinEintraege 	= MIN_MEASURE_INTERVALL;
	Auswahl.Count 			= Ri.messintervall;
	Auswahl.Step 			= 1;
	Auswahl.BigStep 		= 5;
	Auswahl.Roll 			= FALSE;
	Auswahl.Changed 		= TRUE;
	Auswahl.Enable 			= TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Messintervall);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);
		
			// Text schreiben
			page = 4;
			ultoa(Auswahl.Count, buffer1, 10);
			CopyLang(buffer2, txt_s);
			strcat(buffer1, buffer2);
			LCD_WriteString_Medium_Center(buffer1, 0, 127);
		}
	}

	Ri.messintervall = Auswahl.Count;
	Auswahl.Enable = FALSE;
}



/**
* \brief
* Sprache einstellen
* \version
* 1.0 Erste Version
*/
void Sprache(void)
{
	// Auswahl initialisieren
	Auswahl.MaxEintraege = ANZ_SPRACHEN - 1;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = GetLanguage();
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = FALSE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Language);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);
		
			// Text schreiben
			page = 4;
			SetLanguage(Auswahl.Count);
			CopyLang(buffer1, txt_Sprache);
			LCD_WriteString_Medium_Center(buffer1, 0, 127);
		}
	}
	
	// Einstellung speichern
	SRAM_To_EEPROM();
}

/**
* \brief
* Rechnet die Ladeschlussspannung aus
* \version
* Version 1.0
*/
void SchlussspannungAusrechnen(void)
{
	unsigned int Zellenspannung;
	unsigned int ZellenspannungVoll;
	
	Zellenspannung = pgm_read_word(&(CellVoltgeEmpty[Akku.Typ]));
	ZellenspannungVoll = pgm_read_word(&(CellVoltgeFull[Akku.Typ]));
	
	if ((Akku.Zellenzahl * ZellenspannungVoll) > EE_SCHLUSSSPANNUNG_MAX)
	{
		Akku.Zellenzahl = EE_SCHLUSSSPANNUNG_MAX / ZellenspannungVoll;
	}
	
	Akku.Entladespannung = Akku.Zellenzahl * Zellenspannung;
}


/**
* \brief
* Zeigt die Version und das Datum auf dem LCD an
* \version
* Version 1.0
*/
void SplashScreen(void)
{
	unsigned char i;

	// LCD lschen
	ks0108ClearScreen();

	// Koordinaten setzen
	column = 40;
	page = 0;

	// Titel schreiben
	CopyLang(buffer1, txt_Version);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	// Koordinaten setzen
	column = 55;
	page = 4;
	
	LCD_WriteInteger_Medium(SW_MAJOR);
	LCD_WriteString_Medium(".");
	LCD_WriteInteger_Medium(SW_MINOR);

	// Koordinaten setzen
	column = 30;
	page = 6;
	
	LCD_WriteZahl10_Medium(SW_DAY);
	LCD_WriteString_Medium(".");
	LCD_WriteZahl10_Medium(SW_MONTH);
	LCD_WriteString_Medium(".");
	LCD_WriteInteger_Medium(SW_YEAR);

	for (i=0; i<20; i++)
	{
		_delay_ms(100);
	}
}


//*************************************************************
/// \brief
/// Schreibt eine Zeile im Entladungsfenster -> Text: Zahl Einheit 
/// txt:    num unit	(wobei num und unit rechtsbuendig sind)
/// \note
/// Zuvor muss die Position mit den Variabeln page und column
/// festgelegt werden
//*************************************************************
void WriteSettingsNumberLine ( Text txt, unsigned long num, Text unit ) {
	CopyLang(buffer1, txt);
	LCD_WriteString_Small(buffer1);
	LCD_WriteChar_Small(':');
	ultoa(num, buffer1, 10);
	CopyLang(buffer2, unit);
	strcat(buffer1, buffer2);
	LCD_WriteString_Small_Right(buffer1, 127);
}


//*************************************************************
/// \brief
/// Schreibt eine Zeile im Entladungsfenster -> Text: Text
/// txt1:    txt2			(wobei txt2 rechtsbuendig ist)
/// \note
/// Zuvor muss die Position mit den Variabeln page und column
/// festgelegt werden
//*************************************************************
void WriteSettingsStringLine ( Text txt1, Text txt2 ) {
	CopyLang(buffer1, txt1);
	CopyLang(buffer2, txt2);
	LCD_WriteString_Small(buffer1);
	LCD_WriteChar_Small(':');
	LCD_WriteString_Small_Right(buffer2, 127);
}


//*************************************************************
/// \brief
/// Zeigt die Einstellungen zum Entladen an 
/// \note
/// nur Inhalt ohne Frame
//*************************************************************
void ShowDischargeSettingsContent ( void ) {
	unsigned long Voltage;
	unsigned int CellVoltage;
	unsigned char type;
	
	// Zellenspannung auslesen
	CellVoltage = pgm_read_word(&(CellVoltgeFull[Akku.Typ]));
	Voltage = CellVoltage * Akku.Zellenzahl;

	// Akkutyp
	page = 2;
	column = 0;
	switch (Akku.Typ)
	{
		// NiHM
		case 1:
			type = 13;
			break;

		// Pb
		case 2:
			type = 14;
			break;

		// Li-Ion
		case 3:
			type = 40;
			break;
			
		// NiCd
		default:
			type = 12;
			break;
	}
	WriteSettingsStringLine(txt_Akkutyp, type);


	// Zellenzahl
	page = 3;
	column = 0;
	CopyLang(buffer1, txt_Zellenzahl);
	LCD_WriteString_Small(buffer1);
	LCD_WriteChar_Small(':');
	ultoa(Akku.Zellenzahl, buffer1, 10);
		strcat(buffer1, ", ");
		
		ultoa(Voltage, buffer2, 10);
		strcat(buffer1, buffer2);
		
		CopyLang(buffer2, txt_mV);
		strcat(buffer1, buffer2);
	LCD_WriteString_Small_Right(buffer1, 127);


	// Entladestrom
	page = 4;
	column = 0;
	WriteSettingsNumberLine(txt_Entladestrom, Akku.Entladestrom, txt_mA);


	// Schlussspannung
	page = 5;
	column = 0;
	WriteSettingsNumberLine(txt_Schlussspannung, Akku.Entladespannung, txt_mV);


	// Speicherintervall
	page = 6;
	column = 0;
	if (Akku.Speicherintervall != 0)
	{
		WriteSettingsNumberLine(txt_Speicherintervall, Akku.Speicherintervall, txt_s);
	}
	else
	{
		WriteSettingsStringLine(txt_Speicherintervall, txt_aus);
	}
}


//*************************************************************
/// \brief
/// Schreibt die Einstullungen ins Einstellungsframe
//*************************************************************
void ShowRiSettingsContent ( void ) {
	// max. Entladestrom
	page = 2;
	column = 0;
	WriteSettingsNumberLine(txt_Entladestrom, Ri.maxCurrent , txt_mA);
	
	// Intervall
	page = 3;
	column = 0;
	WriteSettingsNumberLine(txt_Messintervall, Ri.messintervall, txt_s);
}


//*************************************************************
/// \brief
/// Zeigt die Einstullungen inkl. Handling der Auswahl
/// Aendern oder Start
/// \param
/// title Textnummer des Titels
/// \param
/// content DischargeSettings oder RiSettings
/// \return
/// AENDERN oder START
//*************************************************************
FrameAnswer ShowSettings ( Text title, SettingsFrame content ) {
	
	// Auswahl initialisieren
	Auswahl.MaxEintraege = 1;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = 0;
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = FALSE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	
	// LCD lschen
	ks0108ClearScreen();

	// Titel schreiben
	page = 0;
	CopyLang(buffer1, title);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	// Titel unterstreichen
	ks0108FillRect(0, 14, 127, 0, BLACK);
	
	// Inhalt schreiben
	switch (content) {
		case DischargeSettings:
			ShowDischargeSettingsContent();
			break;
			
		case RiSettings:
			ShowRiSettingsContent();
			break;
	}
	
	// Strich fr Fusszeile
	ks0108FillRect(0, 55, 127, 0, BLACK);

	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;

			// Alte Fusszeile lschen
			ks0108FillRect(0, 56, 127, 7, WHITE);			

			// Fusszeile schreiben
			page = 7;
			CopyLang(buffer1, txt_Aendern);
			LCD_WriteString_Small_Center(buffer1, 0, 63);

			CopyLang(buffer1, txt_Start);
			LCD_WriteString_Small_Center(buffer1, 64, 127);

			// Button invertieren
			if (Auswahl.Count == Aendern)
			{
				ks0108InvertRect(0, 56, 64, 7);
			}
			else
			{
				ks0108InvertRect(63, 56, 64, 7);
			}
		}
	}

	Auswahl.Enable = FALSE;
	
	return Auswahl.Count;
}


/**
* \brief
* Zeigt die Kopfzeile fuer die Entladezustaende an
* \version
* 1.1 ShowDischargeFrame in Kopf- und Fusszeile aufgeteilt
* 1.0 Erste Version
*/
void ShowDischargeHeadNote(void)
{
	// Tastenentprellung
	OKEntprellen();

	// LCD lschen
	ks0108ClearScreen();

	// Titel schreiben
	page = 0;
	CopyLang(buffer1, txt_Entladen);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	// Titel unterstreichen
	ks0108FillRect(0, 14, 127, 0, BLACK);
}


/**
* \brief
* Zeigt die Fusszeile fuer die Entladezustaende an
* \version
* 1.1 ShowDischargeFrame in Kopf- und Fusszeile aufgeteilt
* 1.0 Erste Version
*/
void ShowDischargeFootNote(void)
{
	// Strich fr Fusszeile
	ks0108FillRect(0, 55, 127, 0, BLACK);

	// Alte Fusszeile lschen
	ks0108FillRect(0, 56, 127, 7, WHITE);			

	// Fusszeile schreiben
	page = 7;
	CopyLang(buffer1, txt_Aendern);
	LCD_WriteString_Small_Center(buffer1, 0, 63);

	CopyLang(buffer1, txt_Beenden);
	LCD_WriteString_Small_Center(buffer1, 64, 127);

	// Button invertieren
	if (Auswahl.Count == 0)
	{
		ks0108InvertRect(0, 56, 64, 7);
	}
	else
	{
		ks0108InvertRect(63, 56, 64, 7);
	}
}


/**
* \brief
* Zeigt die Entladezustaende an
* \version
* 1.0 Erste Version
*/
void ShowDischargeContent(void)
{
	unsigned long Energie;
	unsigned long Kapazitaet;

	Energie = Akku.Energie;
	Energie /= 3600;
	Kapazitaet = Akku.Kapazitaet;
	Kapazitaet /= 3600;

	// Alte Werte lschen
	ks0108FillRect(0, 15, 127, 39, WHITE);

	// Energie
	page = 2;
	column = 0;
	WriteSettingsNumberLine(txt_Energie, Energie, txt_mWh);


	// Kapazitt
	page = 3;
	column = 0;
	WriteSettingsNumberLine(txt_Kapazitaet, Kapazitaet, txt_mAh);


	// Spannung
	page = 4;
	column = 0;
	WriteSettingsNumberLine(txt_Spannung, Akku.Spannung, txt_mV);


	// Strom
	page = 5;
	column = 0;
	WriteSettingsNumberLine(txt_Strom, Akku.Strom, txt_mA);


	// Temperatur
	page = 6;
	column = 0;
	WriteSettingsNumberLine(txt_Temperatur, (CoolerTemp / 10), txt_GradCelsius);


	// Strich fr Fusszeile
	ks0108FillRect(0, 55, 127, 0, BLACK);
}


/**
* \brief
* Zeigt die Fehler meldung fr keine SD-Karte eingelegt
* \version
* 1.0 Erste Version
*/
void NoSDCard(void)
{
	// LCD lschen
	ks0108ClearScreen();

	// Titel schreiben
	page = 2;
	CopyLang(buffer1, txt_SdKarteFehler);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);
	page = 5;
	CopyLang(buffer1, txt_LoggenIstDeaktiviert);
	LCD_WriteString_Small_Center(buffer1, 0, 127);

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// Strich fr Fusszeile
	ks0108FillRect(0, 55, 127, 0, BLACK);
	
	// Fusszeile schreiben
	page = 7;
	CopyLang(buffer1, txt_weiter);
	LCD_WriteString_Small_Center(buffer1, 0, 123);
	ks0108InvertRect(31, 56, 64, 7);

	// Warten bis Taste gedrckt
	while (TasteOK.Kurz == FALSE) {;}

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;
}


//*************************************************************
/// \brief
/// Zeigt an, dass der Entladevorgang oder die 
/// Ri Messung beendet ist
/// \param
/// mode Gibt an, welcher Inhalt angezeigt werden soll
//*************************************************************
void ShowEnd ( EndMode mode ) {
	// LCD lschen
	ks0108ClearScreen();

	// Titel schreiben
	page = 0;
	CopyLang(buffer1, txt_Fertig);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	// Titel unterstreichen
	ks0108FillRect(0, 14, 127, 0, BLACK);

	switch (mode) {
		case EntladenEnd:
			writeEntladenEndContent();
			break;
			
		case RiEnd:
			writeRiEndContent();
			break;
	}


	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// Strich fr Fusszeile
	ks0108FillRect(0, 55, 127, 0, BLACK);
	
	// Fusszeile schreiben
	page = 7;
	CopyLang(buffer1, txt_weiter);
	LCD_WriteString_Small_Center(buffer1, 0, 123);
	ks0108InvertRect(31, 56, 64, 7);

	// Warten bis Taste gedrckt
	while (TasteOK.Kurz == FALSE)
	{
		CoolerTemp = Read_Temp();
	}

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;
}


void writeEntladenEndContent ( void ) {
	unsigned long Energie;
	unsigned long Kapazitaet;

	Energie = Akku.Energie / 3600;
	Kapazitaet = Akku.Kapazitaet / 3600;
	
	// Ladung
	page = 3;
	column = 0;
	WriteSettingsNumberLine(txt_Energie, Energie, txt_mWh);

	// Kapazitt
	page = 4;
	column = 0;
	WriteSettingsNumberLine(txt_Kapazitaet, Kapazitaet, txt_mAh);
	
}

void writeRiEndContent ( void ) {

	// Maximaler Ri
	page = 3;
	column = 0;
	WriteSettingsNumberLine(txt_MaximalerRi, (unsigned long)(Ri.maxRi * 1000), txt_mOhm);
	
	// Mittelwert
	page = 4;
	column = 0;
	WriteSettingsNumberLine(txt_MittlererRi, (unsigned long)(Ri.middleRi * 1000) , txt_mOhm);
	
}


/**
* \brief
* Zeigt an, dass der Kuehler ueberhitz ist und warete bis er wieder kalt ist
* \version
* 1.0 Erste Version
*/
void ShowOverheat(void)
{
	// Ladestrom abschalten
	OCR1B = 0;

	// LCD lschen
	ks0108ClearScreen();

	// Titel schreiben
	page = 2;
	CopyLang(buffer1, txt_Ueberhitzung);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	// Warten bis Temperatur unter die Enschalttemperatur des Lfters gefallen ist
	page = 5;
	CopyLang(buffer2, txt_GradCelsius);
	while (CoolerTemp > FAN_TOFF) 
	{
		if (Zeit.Changed == TRUE)
		{
			Zeit.Changed = FALSE;

			// Temperatur einlesn
			CoolerTemp = Read_Temp();

			// Temperatur schreiben
			ultoa((CoolerTemp / 10), buffer1, 10);
			strcat(buffer1, buffer2);
			ks0108FillRect(0, 40, 127, 8, WHITE);
			LCD_WriteString_Small_Center(buffer1, 0, 127);
		}
	}

	// Entladestrom einstellen
	DAC(Akku.Entladestrom);

	// Frame wieder anzeigen
	ShowDischargeHeadNote();
	ShowDischargeFootNote();
}


/**
* \brief
* Auf Default Settings zuruecksetzen
* \version
* 1.0 Erste Version
*/
void Werkseinstellung(void)
{
	// Sprache
	SetLanguage(EE_SPRACHE_DEF);

	// Akkutyp
	Akku.Typ = EE_AKKUTYP_DEF;

	// Zellenzahl
	Akku.Zellenzahl = EE_ZELLENZAHL_DEF;

	// Entladestrom
	Akku.Entladestrom = EE_ENTLADESTROM_DEF;

	// Schlussspannung
	SchlussspannungAusrechnen();

	// Speicherintervall
	Akku.Speicherintervall = EE_SPEICHERN_DEF;

	// Werte in EEPROM abspeichern
	SRAM_To_EEPROM();
	
	// Kalibrierunge
	Kalibration = 1;
}


/**
* \brief
* Strom einstellen
* \param Current Strom in mA
* \version
* 1.1 Angepasst an Hardware
* 1.0 Erste Version
*/
void DAC(unsigned int Current)
{
	float PWM;

	/*-------------------------------

	 I = Ur / R
	 Ur = I * R
	 Upwm = (I * R) / 2

					I * R
			1024 * -------
					  2
	 PWM = --------------------------
	 		     5000mV

			
	 PWM = 0.18432 * I

			  I
	 PWM = -------
	 		5.425
	-------------------------------*/
	
	// Wenn Current 0 dann ganz abschalten
	if (Current != 0)
	{
		PWM = Current / 5.425;
		OCR1B = (unsigned int) PWM;
	}
	else
	{
		OCR1B = 0;
	}
}


/**
* \brief
* Maximalen Strom ausrechnen
* \version
* 1.0 Erste Version
*/
void MaxStromAusrechnen (void)
{
	unsigned long Current;
	unsigned int CellVoltage;
	unsigned long Voltage;
	
	// Zellenspannung auslesen
	CellVoltage = pgm_read_word(&(CellVoltgeFull[Akku.Typ]));
	Voltage = CellVoltage * Akku.Zellenzahl;

	// Maximalen Strom ausrechnen
	Current = MAX_POWER;
	Current /= Voltage;

	// Auf 10 genau runden
	Current /= 10;
	Current *= 10;

	// berprfen, ob Maximalstrom nicht berschritten
	if (Current > MAX_CURRENT)
	{
		Current = MAX_CURRENT;
	}

	// berprfen, ob gespeicherter Strom nicht zu gross;
	if (Akku.Entladestrom > Current)
	{
		Akku.Entladestrom = Current;
	}
}


/**
* \brief
* Spannung kalibrieren
* \version
* 1.0 Erste Version
*/
void Kalibrierung(void)
{
	unsigned int ADCInt;
	float HWKorr = 1;

	// Auswahl initialisieren
	Auswahl.MaxEintraege = 1;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = 0;
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = FALSE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Kalibrierung);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);

	// Anweisung schreiben
	page = 3;
	CopyLang(buffer1, txt_KlemmenAn5V);
	LCD_WriteString_Small_Center(buffer1, 0, 127);

	// Soll / Ist schreben
	page = 5;
	column = 0;
	CopyLang(buffer1, txt_SollIstKorr);
	LCD_WriteString_Small(buffer1);

	while (TasteOK.Kurz == FALSE)
	{
		if (Zeit.Changed == TRUE)
		{
			Zeit.Changed = FALSE;

			// Istspannungen einlesen
			Set_ADRefInternal();
			_delay_ms(1);
			Set_Channel(VOLTAGE_PINNR);
			ADCInt = ADC_Read();

			// Korrekturen ausrechnen
			HWKorr = SOLLADCHW;
			HWKorr /= ADCInt;

			// Sting Int zusammensetzen
			ultoa(SOLLADCHW, buffer1, 10);
			strcat(buffer1, " / ");

			ultoa(ADCInt, buffer2, 10);
			strcat(buffer1, buffer2);
			strcat(buffer1, " / ");

			dtostrf(HWKorr, 1, 4, buffer2);
			strcat(buffer1, buffer2);

			// Int schreiben
			ks0108FillRect(0, 47, 127, 7, WHITE);
			page = 6;
			column = 0;
			LCD_WriteString_Small(buffer1);

			// Strich fr Fusszeile
			ks0108FillRect(0, 55, 127, 0, BLACK);
		}

		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			
			// Strich fr Fusszeile
			ks0108FillRect(0, 55, 127, 0, BLACK);

			// Alte Fusszeile lschen
			ks0108FillRect(0, 56, 127, 7, WHITE);			

			// Fusszeile schreiben
			page = 7;
			CopyLang(buffer1, txt_Abbrechen);
			LCD_WriteString_Small_Center(buffer1, 0, 63);

			CopyLang(buffer1, txt_Speichern);
			LCD_WriteString_Small_Center(buffer1, 64, 127);

			// Button invertieren
			if (Auswahl.Count == 0)
			{
				ks0108InvertRect(0, 56, 64, 7);
			}
			else
			{
				ks0108InvertRect(63, 56, 64, 7);
			}
		}
	}

	// Korrekturwerte abspeichern
	if (Auswahl.Count == 1)
	{
		Kalibration = HWKorr;
	}

	Auswahl.Enable = FALSE;
	
	// Einstellung speichern
	SRAM_To_EEPROM();
}


/** Tabelle mit den Ladeschlussspannungen pro Zelle [mV] */
const unsigned int CellVoltgeEmpty[] PROGMEM = 
{
	900,		// NiCd
	1000,		// NiMH
	1750,		// Pb
	2500		// Li-Ion
};

/** Tabelle mit den Spannungen pro Zelle (Voll geladen) [mV] */
const unsigned int CellVoltgeFull[] PROGMEM = 
{
	1200,		// NiCd
	1200,		// NiMH
	2000,		// Pb
	3600		// Li-Ion
};


