/**
* \file application.h
* \brief In dieser Datei sind Funktionen und Definitionen fuer die Hauptanwendung die lauft
* \version 
* v 1.4 Li-Ion Unterstuezung hinzugefgt, Ladevorgang kann unterbrochen werden <br>
* v 1.3 Die Temperatur, bei der der Lfter voll dreht ist nun unabhngig von der Maximaltemperatur des Kuhlers <br>
* v 1.2 Spannungen bis 40V sind nun mglich <br>
* v 1.1 Diverse Verbesserungen bei der Darstellung des Menues <br>
* v.1.0 Erste Version
* \date
* Create: 30.07.2008 <br>
* Modify: 14.10.2008
* \author Philipp Kaelin 
*/

#ifndef APPLICATION_H
	#define APPLICATION_H

	//***********
	//* Includes
	//***********
	#include <avr/io.h>
	#include <avr/wdt.h>
	#include <string.h>
	#include <stdlib.h>
	#include <math.h>
	#include "global.h"
	#include "glcd.h"
	#include "tasten.h"
	#include "menue.h"
	#include "eeprom.h"
	#include "logger.h"
	#include "lang.h"


	//************
	//* Variabeln
	//************

	// Tabelle mit den Ladeschlussspannungen pro Zelle [mV]
	extern const unsigned int CellVoltgeEmpty[] PROGMEM;

	// Tabelle mit den Spannungen pro Zelle (Voll geladen) [mV]
	extern const unsigned int CellVoltgeFull[] PROGMEM;


	//*************
	//* Funktionen
	//*************

	// Betriebsauswahl
	void Application ( void );

	// Globale Einstellungen vornehmen
	void GlobalSettings ( void );
	
	// Innenwiderstand des Akku messen
	void RiMessen ( void );

	// Bedienung der Entladefuntionen
	void Entladen ( void );

	// Akkutyp einstellen
	void Akkutyp(void);
	
	// berprft, ob abgeschaltet werden muss
	unsigned char Abschaltung(void);

	// Anzahl der Zellen einstellen
	void Zellenzahl(void);

	// Entladestrom einstellen
	void Entladestrom ( unsigned int maxCurrent, unsigned int *currentVar );

	// Entladeschlussspannung einstellen
	void Schlussspannung(void);

	// Speicherintervall einstellen
	void Speicherintervall(void);
	
	// Messintervall fuer Ri Messen einstellen
	void Messintervall ( void );

	// Rechnet die Ladeschlussspannung aus
	void SchlussspannungAusrechnen(void);

	// Sprache einstellen
	void Sprache(void);

	// Standardeinstellungen
	void Werkseinstellung(void);

	// Zeigt die Version auf dem LCD an
	void SplashScreen(void);

	// Schreibt eine Zeile im Entladungsfenster -> Text: Zahl Einheit 
	void WriteSettingsNumberLine ( Text txt, unsigned long num, Text unit );

	// Schreibt eine Zeile im Entladungsfenster -> Text: Text
	void WriteSettingsStringLine ( Text txt1, Text txt2 );

	// Zeigt die Einstellungen zum Entladen an
	void ShowDischargeSettingsContent ( void );
	
	// Zeigt die Einstullungen inkl. Handling der Auswahl Aendern oder Start
	void ShowRiSettingsContent ( void );
	
	// Zeigt die Einstullungen inkl. Handling der Auswahl Aendern oder Start
	FrameAnswer ShowSettings ( Text title, SettingsFrame content );

	// Zeigt die Kopfzeile fuer die Entladezustaende an
	void ShowDischargeHeadNote(void);

	// Zeigt die Fusszeile fuer die Entladezustaende an
	void ShowDischargeFootNote(void);

	// Zeigt die Entladezustnde an
	void ShowDischargeContent(void);

	// Zeigt die Fehler meldung fr keine SD-Karte eingelegt
	void NoSDCard(void);

	// Zeigt an, dass der Entladevorgang oder Ri Messung beendet ist
	void ShowEnd ( EndMode mode );
	
	
	void writeEntladenEndContent ( void );
	
	
	void writeRiEndContent ( void );

	// Zeigt an, dass der Kuehler ueberhitz ist und warete bis er wieder kalt ist
	void ShowOverheat(void);

	// Strom einstellen
	void DAC(unsigned int Current);
	
	// Maximalen Strom ausrechnen
	void MaxStromAusrechnen (void);
	
	// Spannung kalibrieren
	void Kalibrierung(void);

#endif
