/**
* \file glcd.h
* \brief In dieser Datei sind Funktionen zur Grafischen Darstellung von Texten auf dem LCD
* \version 
* v 1.4 String an den rechten Rand schreiben <br>
* v 1.3 String mittig des LCDs schreiben <br>
* v 1.2 Funktionen fuer Zahlen mit 2 oder 3 Ziffern <br>
* v.1.1 Angepasst fuer KS0108 Controller <br>
* v.1.0 Erste Version fuer einen UC1608 Controller
* \date
* Create: 23.07.2007<br>
* Modify: 09.08.2008
* \author Philipp Kaelin 
*/


#ifndef GLCD_H
	#define GLCD_H

	//***********
	//* Includes
	//***********
	#include <avr/pgmspace.h>
	#include <stdlib.h>
	#include <string.h>
	#include "ks0108.h"


	//**********
	//* Defines
	//**********
	#define LEERZEICHEN			32
	#define LEN_LEERZEICHEN		2


	//************
	//* Variabeln
	//************
	/** Spalte, die zum schreiben verwendet werden soll */
	extern unsigned char column;
	
	/** Page die zum schreiben verwendet werden soll*/
	extern unsigned char page;


	//*************
	//* Funktionen
	//*************

	// Setzt eine Adresse Column 0-127, page 0-7
	void LCD_SetAddress(unsigned char column, unsigned char page);

	// Schreibt einen Character in Mittlerer Grsse
	void LCD_WriteChar_Medium(char c);

	// Schreibt einen String in Mittlerer Grsse
	void LCD_WriteString_Medium(char *data);

	// Schreibt einen String in mittlerer Schrift in dei Mitte des LCD
	void LCD_WriteString_Medium_Center(char *data, unsigned char LowerBoun, unsigned char UpperBound);

	// Gibt die Lnge in Pixeln eines Strings zurck
	unsigned char GetLength_String_Medium(char *data);

	// Schreibt eine Integer Zahl in Mittlerer Grsse
	void LCD_WriteInteger_Medium(int data);

	// Schreibt eine Integer Zahl in mittlerer Schrift in die Mitte aufs LCD
	void LCD_WriteInteger_Medium_Center(int data, unsigned char LowerBound, unsigned char UpperBound);

	// Schreibt eine Zahl mit 2 Ziffern ggf. 0 voran
	void LCD_WriteZahl10_Medium(int data);

	// Schreibt eine Zahl mit 3 Ziffern ggf. 0 voran
	void LCD_WriteZahl100_Medium(int data);	

	// Schreibt einen Character in Kleiner Grsse
	void LCD_WriteChar_Small(char c);

	// Schreibt einen String in Kleiner Grsse
	void LCD_WriteString_Small(char *data);

	// Schreibt einen String in kleiner Schrift in dei Mitte des LCD
	void LCD_WriteString_Small_Center(char *data, unsigned char LowerBound, unsigned char UpperBound);

	// Schreibt einen String in kleiner Schrift an den rechten Rand des LCD
	void LCD_WriteString_Small_Right(char *data, unsigned char RightBound);

	// Gibt die Lnge in Pixeln eines Strings zurck
	unsigned char GetLength_String_Small(char *data);

	// Gibt die Lnge in Pixeln eines Chars zurck
	unsigned char GetLength_Char_Small(char c);

	// Schreibt eine Integer Zahl in Kleiner Grsse
	void LCD_WriteInteger_Small(int data);
	
	// Zeichent einen Fortschrittbalken auf das LCD
	void writeBarGraph ( unsigned char percentage, unsigned char line );
	
#endif
