/*********
 logger.c
*********/


//***********
//* Includes
//***********
#include "logger.h"

unsigned char by1 = TAB;
unsigned char by2 = KLAMMERAUF;
unsigned char by3 = KLAMMERZU;

/**
* \brief
* Ueberprueft ob eine SDKarte vorhanden ist
* \return TRUE wenn erfolgreich
* \version
* 1.0 Erste Version
*/
unsigned char SD_CheckCard(void)
{
	if(GetDriveInformation() == F_OK)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


/**
* \brief
* Erstellt die Datei zum Loggen
* \return TRUE wenn erfolgreich
* \version
* 1.0 Erste Version
*/
unsigned char SD_CreateLogFile(void)
{
	// Datei lschen, falls sie schon existiert
	Remove(Filename);
 
 	// Datei zum schreiben ffnen
 	if(Fopen(Filename, F_WRITE) != F_OK) 
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


/**
* \brief
* Schliesst die Datei zum Loggen
* \version
* 1.0 Erste Version
*/
void SD_CloseFlie(void)
{
	Fclose();
}


/**
* \brief
* Haengt einen String an die Logdatei an
* \param *Line Pointer auf einen String im RAM
* \param NewLine CrLf schreiben (TRUE / FALSE)
* \return TRUE wenn erfolgreich
* \version
* 1.0 Erste Version
*/
unsigned char SD_AppendData(char *Line, unsigned char NewLine)
{
	unsigned char result = FALSE;
	unsigned char by;

	// String schreiben
	while (*Line)
	{
		by = *Line;
		result = Fwrite(&by, 1);
		Line++;
	}

	// CrLf schreiben, falls ntig
	if (NewLine == TRUE)
	{
		by = '\r';
		result = Fwrite(&by,1);
		by = '\n';
		result = Fwrite(&by,1);
	}

	return result;
}


/**
* \brief
* Haengt eine Nummer an die Logdatei an
* \param Number Zahl (unsigned long)
* \param NewLine CrLf schreiben (TRUE / FALSE)
* \return TRUE wenn erfolgreich
* \version
* 1.0 Erste Version
*/
unsigned char SD_AppendNumber(unsigned long Number, unsigned char NewLine)
{
	unsigned char result;
	char buffer[10];

	// Long in String umwandeln
	ultoa(Number, buffer, 10);

	// String schreiben
	result = SD_AppendData(buffer, NewLine);

	return result;
}


/**
* \brief
* Schreibt eine Leerzeile in die Logdatei
* \return TRUE wenn erfolgreich
* \version
* 1.0 Erste Version
*/
unsigned char SD_NewLine(void)
{
	unsigned char result;
	unsigned char by;

	by = '\r';
	result = Fwrite(&by,1);
	by = '\n';
	result = Fwrite(&by,1);

	return result;
}


/**
* \brief
* Schreibt den Header zu den Logdaten in die Datei
* \return TRUE wenn erfolgreich
* \version
* 1.1 Einheiten korrigiert
* 1.0 Erste Version
*/
unsigned char SD_WriteHeader(void)
{
	unsigned char result;

	// Akkutyp schreiben
	CopyLang(buffer1, txt_Akkutyp);
	result = SD_AppendData(buffer1, FALSE);
	result = Fwrite(&by1, 1);
	switch(Akku.Typ)
	{
		case NICD:
			CopyLang(buffer1, txt_NiCd);
			break;

		case NIMH:
			CopyLang(buffer1, txt_NiMH);
			break;

		case BLEI:
			CopyLang(buffer1, txt_Pb);
			break;

		case LIION:
			CopyLang(buffer1, txt_LiIon);
			break;
			
	}
	result = SD_AppendData(buffer1, TRUE);

	WriteTextAndNumberLine(6, Akku.Zellenzahl);
	WriteTextAndNumberLine(8, Akku.Entladespannung);
	WriteTextAndNumberLine(7, Akku.Entladestrom);
	WriteTextAndNumberLine(9, Akku.Speicherintervall);

	// Leerzeile
	result = SD_NewLine();

	// Beschreibung
	WriteFormattedTextAndUnit(30, 17);
	WriteFormattedTextAndUnit(23, 16);
	WriteFormattedTextAndUnit(24, 15);
	WriteFormattedTextAndUnit(31, 36);
	WriteFormattedTextAndUnit(21, 37);

	result = SD_NewLine();

	return result;
} 


/**
* \brief
* Schreibt die Logdaten in die Datei
* \return TRUE wenn erfolgreich
* \version
* 1.0 Erste Version
*/
unsigned char SD_WriteLog(void)
{
	unsigned char result;
	unsigned char by;

	by = TAB;

	// TimeStamp
	SD_AppendNumber(Akku.Entladezeit, FALSE);
	result = Fwrite(&by, 1);

	// Spannung
	SD_AppendNumber(Akku.Spannung, FALSE);
	result = Fwrite(&by, 1);

	// Strom
	SD_AppendNumber(Akku.Strom, FALSE);
	result = Fwrite(&by, 1);

	// Kapazitaet
	SD_AppendNumber(Akku.Kapazitaet, FALSE);
	result = Fwrite(&by, 1);

	// Energie
	SD_AppendNumber(Akku.Energie, TRUE);

	return result;
}


/**
* \brief
* Name der Logdatei einstellen
* \version
* 1.0 Erste Version
*/
void LogName(void)
{
	// Auswahl initialisieren
	Auswahl.MaxEintraege = 99;
	Auswahl.MinEintraege = 0;
	Auswahl.Count = LogCount;
	Auswahl.Step = 1;
	Auswahl.BigStep = 5;
	Auswahl.Roll = TRUE;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	// LCD lschen
	ks0108ClearScreen();
		
	// Titel
	page = 0;
	CopyLang(buffer1, txt_Dateiname);
	LCD_WriteString_Medium_Center(buffer1, 0, 127);


	while (TasteOK.Kurz == FALSE)
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;

			// Alte Zahl lschen
			ks0108FillRect(0, 30, 127, 20, WHITE);

			// Dateinamen erstellen
			SetLogName(Auswahl.Count);
		
			// Text schreiben
			page = 4;
			LCD_WriteString_Medium_Center((char*) Filename, 0, 127);
		}
	}

	Auswahl.Enable = FALSE;
	LogCount = Auswahl.Count;
}


/**
* \brief
* Name der Logdatei in Filename String schreiben
* \param
* Counter DateiCounter
* \version
* 1.0 Erste Version
*/
void SetLogName(unsigned char Counter)
{
	ultoa(Counter, buffer1, 10);

	Filename[0] = 'l';
	Filename[1] = 'o';
	Filename[2] = 'g';

	if (Counter < 10)
	{
		Filename[3] = '0';
		Filename[4] = *buffer1;
	}
	else
	{
		Filename[3] = *buffer1;
		Filename[4] = *(buffer1 + 1);
	}

	Filename[5] = '.';
	Filename[6] = 't';
	Filename[7] = 'x';
	Filename[8] = 't';
	Filename[9] = 0;
}


unsigned char WriteTextAndNumberLine ( Text txt, unsigned long num ) {
	unsigned char result;

	CopyLang(buffer1, txt);
	result = SD_AppendData(buffer1, FALSE);
	result = Fwrite(&by1, 1);
	SD_AppendNumber(num, TRUE);

	return result;
}


unsigned char WriteFormattedTextAndUnit ( Text txt, Text unit ) {
	unsigned char result;

	CopyLang(buffer1, txt);
	result = SD_AppendData(buffer1, FALSE);
	result = Fwrite(&by2, 1);
	CopyLang(buffer1, unit);
	result = SD_AppendData((buffer1 + 1), FALSE);
	result = Fwrite(&by3, 1);
	result = Fwrite(&by1, 1);

	return result;
}
