/**
* \file logger.h
* \brief In dieser Datei sind Funktionen und Definitionen zum Loggen der Daten 
* \version 
* v.1.3 Li-Ion hinzugefuegt, SetLogName zu SetLogName umbenannt um verwechslung auszuschliessen <br>
* v.1.2 Dateiname einstellbar <br>
* v.1.1 Daten durch Tabulator trennen anstatt Semikolon <br>
* v.1.0 Erste Version
* \date
* Create: 14.08.2008 <br>
* Modify: 14.10.2008
* \author Philipp Kaelin 
*/

#ifndef LOGGER_H
	#define LOGGER_H

	//***********
	//* Includes
	//***********
	#include <stdlib.h>
	#include <avr/io.h>
	#include "global.h"
	#include "dos.h"
	#include "glcd.h"
	#include "lang.h"


	//**********
	//* Defines
	//**********
	#define TAB			0x09
	#define KLAMMERAUF	0x5b
	#define KLAMMERZU	0x5d	



	//*************
	//* Funktionen
	//*************

	// Ueberprueft ob eine SDKarte vorhanden ist
	unsigned char SD_CheckCard(void);

	// Erstellt die Datei zum Loggen
	unsigned char SD_CreateLogFile(void);

	// Schliesst die Datei zum Loggen
	void SD_CloseFlie(void);

	// Haengt einen String an die Logdatei an
	unsigned char SD_AppendData(char *Line, unsigned char NewLine);

	// Haengt eine Nummer an die Logdatei an
	unsigned char SD_AppendNumber(unsigned long Number, unsigned char NewLine);

	// Schreibt eine Leerzeile in die Logdatei
	unsigned char SD_NewLine(void);

	// Schreibt den Header zu den Logdaten in die Datei
	unsigned char SD_WriteHeader(void);

	// Schreibt die Logdaten in die Datei
	unsigned char SD_WriteLog(void);

	// Name der Logdatei einstellen
	void LogName(void);	

	// Name der Logdatei in Filename String schreiben
	void SetLogName(unsigned char Counter);


	unsigned char WriteTextAndNumberLine ( Text txt, unsigned long num );


	unsigned char WriteFormattedTextAndUnit ( Text txt, Text unit );
#endif
