/*********
 KS0108.c
**********/


//***********
//* Includes
//***********
#include "menue.h"


//************
//* Variabeln
//************

/** Tabelle mit den Pointern zu der Langunagetable */
const unsigned char string_table[] PROGMEM =
{
	/* Einstellungen Entladen*/
	txt_Version,				// --> Einstellungen
	txt_Akkutyp,				// Akkutyp
	txt_Zellenzahl,				// Zellenzahl
	txt_Entladestrom,			// Entladestrom
	txt_Schlussspannung,		// Schlussspannung
	txt_Speicherintervall,		// Speicherintervall
	txt_Dateiname,				// Dateiname
	txt_Werkseinstellungen,		// Werkseinstellung
	txt_Zurueck,				// zurck, muss als letztes stehen
	
	/* Betriebswahl */
	txt_Betriebswahl,			// --> Betriebswahl
	txt_AkkuEntladen,			// Akku Entladen
	txt_RiMessen,				// RI messen
	txt_Einstellungen,			// Globale Einstellungen

	/* Globale Einstellungen */
	txt_Einstellungen,			// --> Einstellungen
	txt_Language,				// Language
	txt_Kalibrierung,			// Kalibrierung
	txt_Zurueck,				// zurck, muss als letztes stehen
	
	/* Einstellungen Ri */
	txt_Einstellungen,			// --> Einstellungen
	txt_Entladestrom,			// max. Entladestrom
	txt_Messintervall,			// Messintervall
	txt_Zurueck,				// zurck, muss als letztes stehen
};


/** Tabelle mit den Anfangspositionen der Menues */
const unsigned char start_table[] PROGMEM =
{
	0, 9, 13, 17
};


/** Tabelle mit den Endpositionen der Menues */
const unsigned char stop_table[] PROGMEM =
{
	8, 12, 16, 20
};


/**
* \brief
* Zeichnet das Menue aufs LCD
* \param Menue Angabe, welches Menue gezeichnet werden soll. Dies entspricht dem Tabellenindex der Anfags und Stoppwerte
* \param LastState Angabe, welches der letzte markierte Eintrag war
* \return
* Wenn abgebrochenwurde wird 0 zurueckgegeben, ansonsten die Nummer des Eintrags, der ausgewaehlt wurde.
* \version
* 1.5 Roll Ausaahl implementiert
* 1.4 Fheler bei der Laengenberechnung und beenden nach 60s behoben
* 1.3 Fehler korrigiert bei mehr als 2 Pages
* 1.2 Springt bei Aufruf zum letzten markierten Eintrag 
* 1.1 Fehler korrigiert bei mehreren Menues
* 1.0 Erste Version
*/
unsigned char DrawMenue(unsigned char Menue, unsigned char LastState, unsigned char Roll)
{
	unsigned char PageStart;
	unsigned char PageStop;
	unsigned char MenuePage;
	unsigned char TableStart;
	unsigned char TableStop;
	unsigned char Anzahl;
	unsigned char tempbyte;
	unsigned char SBStart;
	unsigned char SBLenght;

	// Lngen auslesen
	TableStart = pgm_read_byte(&start_table[Menue]);
	TableStop = pgm_read_byte(&stop_table[Menue]);
	Anzahl = TableStop - TableStart;	

	// Page Daten einstellen
	PageStart = 1;
	PageStop = 3;
	MenuePage = 1;

	// berprfen, ob LastState gltig
	if ((LastState > Anzahl) || (LastState < 1))
	{
		LastState = 1;
	}

	// Auswahl initialisieren
	Auswahl.MaxEintraege = Anzahl;
	Auswahl.MinEintraege = 1;
	Auswahl.Count = LastState;
	Auswahl.Step = 1;
	Auswahl.BigStep = 1;
	Auswahl.Roll = Roll;
	Auswahl.Changed = TRUE;
	Auswahl.Enable = TRUE;
	
	// Counter zurcksetzen
	Zeit.totSekunden = 0;

	// Men schreiben
	Auswahl.Changed = TRUE;

	// Tastenentprellung
	OKEntprellen();
	TasteOK.Kurz = FALSE;

	while ((TasteOK.Kurz == FALSE) && (Zeit.totSekunden < 60))
	{
		if (Auswahl.Changed == TRUE)
		{
			Auswahl.Changed = FALSE;
			Zeit.totSekunden = 0;

			// berprfen, ob jetztiger Eintrag nicht grsser als Seitenende
			while (Auswahl.Count > PageStop)
			{
				PageStart += 3;
				PageStop += 3;
			}

			// berprfen, ob jetztiger Eintrag nicht grsser als Seitenanfang
			while (Auswahl.Count < PageStart)
			{
				PageStart -= 3;
				PageStop -= 3;
			}

			// MenuePage aus Seitenanfang ausrechnen
			tempbyte = PageStart;
			MenuePage = 0;
			while(tempbyte > 1)
			{
				MenuePage++;
				tempbyte -= 3;
			}

			// Eintrge schreiben
			DrawMenueEntries(Menue, PageStart, TableStart, TableStop);

			// Neuer Eintrag invertieren
			EintragInvertieren(Auswahl.Count - (PageStart - 1));

			// Scrollbalken zeichnen
			SBLenght = 48 / Auswahl.MaxEintraege;
			SBStart = SBLenght * (Auswahl.Count - 1);
			SBStart += 16;
			SBLenght--;

			ks0108FillRect(124, SBStart, 2, SBLenght, BLACK);
		}
	}

	// Rckgabe des Wert (0 falls ESC gedrckt)
	if (TasteOK.Kurz == TRUE)
	{
		tempbyte = Auswahl.Count;
	}
	else
	{
		tempbyte = 0;
	}

	return tempbyte;
}


/**
* \brief
* Zeichnet die Eintraege des Menues aufs LCD
* \param Menue Tabellenindex des Menues <br>
* \param First Welches der Erste eintrag im Menue ist <br>
* \param Start Index, wo das Menue im Array anfaengt <br>
* \param Stop Index, wo das Menue im Array aufhoert <br>
* \version
* 1.4 Warnings vermieden durch Casting bei strcpy_P
* 1.3 Titel in der Mitte schreiben
* 1.2 Fuer mehrere Sprachen abgeaendert und in der String Table wird nur noch der Index auf eine LangTable geholt
* 1.1 Scrollbalken hinzugefuegt
* 1.0 Erste Version
*/
void DrawMenueEntries(unsigned char Menue, unsigned char First, unsigned char Start, unsigned char Stop)
{
	Text Entry;
	unsigned char i;
	unsigned char Last;
	unsigned char Anzahl;
	char buffer[20];

	Anzahl = Stop - Start;

	// LCD lschen
	ks0108ClearScreen();

	// Koordinaten zurcksetzen
	column = 0;
	page = 0;

	// Titel schreiben
	Entry = pgm_read_byte(&(string_table[Start]));
	CopyLang(buffer, Entry);
	LCD_WriteString_Medium_Center(buffer, 0, 127);

	// Strich unter dem Titel
	ks0108FillRect(0, 14, 127, 0, BLACK);

	// Senkrechter Strich fr Scrollbalken
	ks0108FillRect(122, 15, 0, 48, BLACK);



	Last = First + 2;
	if (Last > Anzahl)
	{
		Last = Anzahl;
	}

	page = 2;

	// Eintraege schreiben
	for (i=First; i<(Last + 1); i++)
	{
		column = 1;

		Entry = pgm_read_byte(&(string_table[i + Start]));
		CopyLang(buffer, Entry);
		LCD_WriteString_Medium(buffer);
		page += 2;
	}
}



/**
* \brief
* Invertiert einen Eintrag im Menue
* \param
* Count Welcher Eintrag invertiert werden soll (1 bis 3))
* \version
* 1.1 Geaendert fuer Scrollbalken
* 1.0 Erste Version
*/
void EintragInvertieren(unsigned char Count)
{
	unsigned char y;
	
	y = Count * 15;
	y += Count;
	ks0108InvertRect(0, y, 121, 15);
}
