/**
* \file adc.h
* \brief In dieser Datei sind Funktionen und Definitionen rund um den AD-Wandler
* \version 
* v 1.4 Die Spannung wird mit dem Kalibrierungsfaktor korrigiert
* v 1.3 Spannungen bis 40V koennen jetzt eingelesen werden
* v 1.2 Verwendet die Interne 2.56V Referenz und die 5V AVCC um genauer zu messen <br>
* v 1.1 Die Referenz kann zwischen Externer und AVCC gewaehlt werden <br>
* v.1.0 Erste Version
* \date
* Create: 28.07.2008
* Modify: 19.10.2008
* \author Philipp Kaelin 
*/

#ifndef ADC_H
	#define ADC_H

	//***********
	//* Includes
	//***********
	#include <avr/io.h>
	#include <util/delay.h>
	#include "global.h"


	//**********
	//* Defines
	//**********
	#define CURRENT_DDR			DDRA
	#define CURRENT_PORT		PORTA
	#define CURRENT_PIN			PINA
	#define CURRENT_PINNR		PA0

	#define VOLTAGE_DDR			DDRA
	#define VOLTAGE_PORT		PORTA
	#define VOLTAGE_PIN			PINA
	#define VOLTAGE_PINNR		PA1

	#define TEMP_DDR			DDRA
	#define TEMP_PORT			PORTA
	#define TEMP_PIN			PINA
	#define TEMP_PINNR			PA2

	// Referenzen
	#define INT_REF				1
	#define AVCC_REF			0

	#define INT_VOLTAGE			2560
	#define AVCC_VOLTAGE		5000




	//*************
	//* Funktionen
	//*************

	// Liest einen Analogeingang ein. Die Referenz und der Kanal und  muss vorher gewhlt werden
	unsigned int ADC_Read(void);

	// Setzt die Kanalnummer
	void Set_Channel(unsigned char mux);

	// Whlt die interne Referenzspannung aus
	void Set_ADRefInternal(void);

	// Whlt die Betriebsspannung AVCC als Referenzspannung aus
	void Set_ADRefVCC(void);

	// Liest die Temperatur ein
	unsigned int Read_Temp(void);

	// Liest die Spannung ein
	unsigned long Read_Voltage(void);

	// Liest den Strom ein
	unsigned int Read_Current(void);
	
#endif
