/**
* \file eeprom.h
* \brief In dieser Datei sind Funktionen die zum lesen und scheiben des AVR Internen EEPROMS sind
* \note Waehrend dem schreiben oder lesen sind die Interrupts deaktiviert
* \version 
* v 1.3 Kalibrierungsfaktor wird abgespeichert
* v 1.2 Die Entladeschlussspannung wird nicht mehr im EEPROM abgespeichert, sonder errechnet.
* v. 1.1 Interrupts waehrend dem Zugriff aufs EEPROM deaktivieren <br>
* v. 1.0 Erste Version aus Datenblatt
* \date
* Create: 30.03.2008 <br>
* Modify: 19.10.2008
* \author Philipp Kaelin 
*/

#ifndef EEPROM_H
	#define EEPROM_H

	//***********
	//* Includes
	//***********
	#include <avr/io.h>
	#include <avr/interrupt.h>
	#include "global.h"
	#include "application.h"
	
	//**********
	//* Defines
	//**********
	typedef union {
 		float value;
 		unsigned char byte_val[4];
	} float2byte;
	
	typedef union {
		float 			f_value;
		unsigned long	l_value;
	} float2long;

	
	//*************
	//* Funktionen
	//*************

	// Liest die Daten aus dem EEPROM und speichert sie ins RAM
	void EEPROM_To_SRAM(void);

	// Speichert die Daten vom SRAM ins EEPROM
	void SRAM_To_EEPROM(void);
	
	// Writing Functions
	void writeByte2eeprom 		( unsigned int adress, unsigned char val );
	void writeInteger2eeprom 	( unsigned int adress, unsigned int val );
	void writeLong2eeprom 		( unsigned int adress, unsigned long val );
	void writeFloat2eeprom 		( unsigned int adress, float val );

	// Reading Functions
	unsigned char 	readByteFromEeprom 		( unsigned int adress, unsigned char lBound, unsigned char uBound, unsigned char def );
	unsigned int 	readIntegerFromEeprom	( unsigned int adress, unsigned int lBound, unsigned int uBound, unsigned int def );
	unsigned long 	readLongFromEeprom 		( unsigned int adress, unsigned long lBound, unsigned long uBound, unsigned long def );
	float 			readFloatFromEeprom 	( unsigned int adress, float lBound, float uBound, float def );

	// Hardware Funtions
	unsigned char EEPROM_read(unsigned int uiAddress);
	void EEPROM_write(unsigned int uiAddress, unsigned char ucData);
#endif
