/*******
 init.c
********/


//***********
//* Includes
//***********
#include "init.h"

/**
* \brief
* Initialisiert die Hardwarekonfiguration des LCD's
* \version
* 1.0 Erste Version
*/
void ks0108InitHW(void)
{	
	// Initialisiert die Level
	LCD_DI_PORT &= ~(1 << LCD_DI_PINNR);
	LCD_RW_PORT &= ~(1 << LCD_RW_PINNR);
	LCD_EN_PORT &= ~(1 << LCD_EN_PINNR);
	LCD_CS1_PORT &= ~(1 << LCD_CS1_PINNR);
	LCD_CS2_PORT &= ~(1 << LCD_CS2_PINNR);
	LCD_RESET_PORT &= ~(1 << LCD_RESET_PINNR);
	
	// Initialisiert die Datenrichtung
	LCD_DI_DDR |= (1 << LCD_DI_PINNR);
	LCD_RW_DDR |= (1 << LCD_RW_PINNR);
	LCD_EN_DDR |= (1 << LCD_EN_PINNR);
	LCD_CS1_DDR |= (1 << LCD_CS1_PINNR);
	LCD_CS2_DDR |= (1 << LCD_CS2_PINNR);
	LCD_RESET_DDR |= (1 << LCD_RESET_PINNR);
	
	// Initialisiert die Level
	LCD_Write_Out(0x00);
	
	// Initialisiert die Datenrichtung
	LCD_Dir_Out();
}


/**
* \brief
* Initialisiert die Hardwarekonfiguration des Incrementalencoders
* \version
* 1.0 Erste Version
*/
void EncoderInit(void)
{
	//Phase A als Eingang mit PullUp
	ENCODER_A_DDR &= ~(1 << ENCODER_A_PINNR);
	ENCODER_A_PORT |= (1 << ENCODER_A_PINNR);

	// Phase B als Eingang mit PullUp
	ENCODER_B_DDR &= ~(1 << ENCODER_B_PINNR);
	ENCODER_B_PORT |= (1 << ENCODER_B_PINNR);
}


/**
* \brief
* Initialisiert die Hardwarekonfiguration des Tasters
* \version
* 1.0 Erste Version
*/
void TasteInit(void)
{
	//Taster als Eingang mit PullUp
	TASTER_DDR &= ~(1 << TASTER_PINNR);
	TASTER_PORT |= (1 << TASTER_PINNR);
}


/**
* \brief
* Initialisiert die Analogeingaenge
* \version
* 1.0 Erste Version
*/
void ADPinsInit(void)
{
	// PullUp's ausschalten
	CURRENT_PORT &= ~(1 << CURRENT_PINNR);
	VOLTAGE_PORT &= ~(1 << VOLTAGE_PINNR);
	TEMP_PORT &= ~(1 << TEMP_PINNR);

	// Pins als Eingnge
	CURRENT_DDR &= ~(1 << CURRENT_PINNR);
	VOLTAGE_DDR &= ~(1 << VOLTAGE_PINNR);
	TEMP_DDR &= ~(1 << TEMP_PINNR);	
}


/**
* \brief
* Initialisiert die PWM Ausgangspins
* \version
* 1.0 Erste Version
*/
void PWMPinsInit(void)
{
	// PullUp's ausschalten
	PWM_PORT &= ~(1 << PWM_PINNR);
	FAN_PORT &= ~(1 << FAN_PINNR);

	// Pins als Ausgnge
	PWM_DDR |= (1 << PWM_PINNR);
	FAN_DDR |= (1 << FAN_PINNR);
}


/**
* \brief
* Initialisiert den AD Konverter
* \version
* 1.0 Erste Version
*/
void ADC_Init(void)
{
	// Prescaler auf 128
	// ADC-Frequenz = 8MHz / 128 = 62.5 kHz
	ADCSRA |= ((1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0));

	// ADC Enablen
	ADCSRA |= (1 << ADEN);
}


/**
* \brief
* Initialisiert den Timer2 fuer die 1ms Zeitbasis
* \version
* 1.0 Erste Version
*/
void Timer2_Init(void)
{
	/*-------------------------------
	 Durch den CTC Mode wird nicht
	 bis zu TOP gezhlt, sondern
	 nur bis OCR2 
	-------------------------------*/
	TCCR2 |= (1 << WGM21);


	/*-------------------------------
	 1ms / (1 / 8MHz) = 8'000 Clocks
	 8'000 Clk / 256 Clk = 31.25
	    --> Prescaler min 31.25
	-------------------------------*/
	// Prescaler = 32
	TCCR2 |= ((1 << CS21) | (1 << CS20));


	/*-------------------------------
	 8000 Clocks / 32 = 250 Clocks
	-------------------------------*/
	OCR2 = 250;


	// Output Compare Match Interrupt
	TIMSK |= (1 << OCIE2);
}


/**
* \brief
* Initialisiert den Timer1 fuer die PWM Ausgnge
* \version
* 1.0 Erste Version
*/
void Timer1_Init(void)
{
	/*-------------------------------
	 Frequenzberechnung:
	 Die PWM frequenz muss etwa 2
	 Dekaden grsser sein als die
	 des Tiefpasses zur glttung.


	 fg = 1 / (2 * Pi * R * C)
	 fg = 1 / (2 * 3.14 * 10k * 100n)
	 fg = 159Hz

	 Daher muss die PWM Frequenz
	 etwa 15kHz betragen

	        ===============

     Periodenzeit des Ausgangssignals
	 Tout = 1 / fout = 1 / 15kHz 
	 Tout = 66us

	 Periodenzeit der Timerfrequenz
	 (2^10 mal mehr als Tout)
	 Tpwm = Tout / (2^10)
	 Tpwm = 64ns

	 Periodenzeit eines Taktzyklus
	 Tosc = 1 / fosc = 1 / 8MHz
	 Tosc = 125ns

	 Prescaler = Tpwm / Tosc
	 Prescaler = 0.51
	 
	        ===============

	 Da der Prescaler kleiner als 1
	 ist mssen wir uns mit der Ferq.
	 begngen, die er mit 1 raus gibt
	 (ca 7.5 kHz) :-(
	-------------------------------*/
	

	// Prescaler = 1
	TCCR1B |= (1 << CS10);


	// Fast PWM 10bit
	TCCR1A |= ((1 << WGM12) | (1 << WGM11) | (1<<WGM10));

	// Ausgang A Nicht Invertierend
	TCCR1A |= (1 << COM1A1);

	// Ausgang B Nicht Invertierend
	TCCR1A |= (1 << COM1B1);
}
