/**********
 encoder.c
***********/


//***********
//* Includes
//***********
#include "encoder.h"

/** Zeit, bei der der letzte Step war */
volatile unsigned int		enc_time;

/**  -128 ... 127 */
volatile char				enc_delta;

/** Encoder Zwischenzaehler -4 bis 4*/
volatile unsigned char		enc_Zcount = 100;

/** Drehrichtung des Encoders */
volatile unsigned char		richtung = UP;


/**
* \brief
* Liest den Zustand des Encoders ein und ruft ggf. die Funktionen zum Auf- und Abzaehlen auf
* \version
* 1.1 Fehler bei Drehrichtungswechsel behoben
* 1.0 Erste Version
*/
void EncoderEinlesen(void)
{
	// Encoder einlesen	
	static char enc_last = 0x01;
	char enc_i = 0;
	enc_delta = 100;
	
	enc_time++;
	if (enc_time > BIGSTEPMAXMS) 
	{
		enc_time = BIGSTEPMAXMS;	
	}

	if (ENCODER_A_PIN & (1 << ENCODER_A_PINNR))
	{
		enc_i = 1;
	}

	if (ENCODER_B_PIN & (1 << ENCODER_B_PINNR))
	{
		enc_i ^= 3;
	}

	enc_i -= enc_last;

	if( enc_i & 1 )
	{
		enc_last += enc_i;

		enc_delta += (enc_i & 2) - 1;
	}

	if (enc_delta == 101)
	{
		if (richtung == UP)
		{
			enc_Zcount++;
		}
		else
		{
			enc_Zcount = 101;
		}

		richtung = UP;
	}
	if (enc_delta == 99)
	{
		if (richtung == DOWN)
		{
			enc_Zcount--;
		}
		else
		{
			enc_Zcount = 99;
		}

		richtung = DOWN;
	}
	
	if (enc_Zcount == 104)
	{
		AuswahlUp();
		enc_Zcount = 100;
	}
	if (enc_Zcount == 96)
	{
		AuswahlDown();
		enc_Zcount = 100;
	}
}


/**
* \brief
* Inkrementiert die Auswahl unter Beachtung der oberen Grenze
* \version
* 1.0 Erste Version
*/
void AuswahlUp(void)
{
	// Counter hat gendert
	Auswahl.Changed = TRUE;

	// Falls Zeit zwischen zewi Steps kleiner 
	if (enc_time < BIGSTEPMAXMS)
	{
		// Conter mit BigStep inkrementieren
		Auswahl.Count += Auswahl.BigStep;
	}
	else
	{
		// Counter normal inkrementieren
		Auswahl.Count += Auswahl.Step;
	}
	

	// Falls berlauf, dann zurcksetzen
	if (Auswahl.Count > Auswahl.MaxEintraege)
	{
		if (Auswahl.Roll == TRUE)
		{
			Auswahl.Count = Auswahl.MinEintraege;
		}
		else
		{
			Auswahl.Count = Auswahl.MaxEintraege;
			Auswahl.Changed = FALSE;
		}
	}
	
	// Zeit des letzten Schritts merken
	enc_time = Zeit.totSekunden;
}


/**
* \brief
* Dekrementiert die Auswahl unter Beachtung der unteren Grenze
* \version
* 1.0 Erste Version
*/
void AuswahlDown(void)
{
	// Counter hat gendert
	Auswahl.Changed = TRUE;

	// Falls Zeit zwischen zewi Steps kleiner 
	if (enc_time < BIGSTEPMAXMS) 
	{
		// Counter mit BigStep dekrementieren
		Auswahl.Count -= Auswahl.BigStep;
	}
	else
	{
		// Counter normal dekrementieren
		Auswahl.Count -= Auswahl.Step;	
	}
	
	
	// Zeitzhler zurcksetzen
	enc_time = 0;

	// Falls berlauf, dann zurcksetzen
	if (Auswahl.Count < Auswahl.MinEintraege)
	{
		if (Auswahl.Roll == TRUE)
		{
			Auswahl.Count = Auswahl.MaxEintraege;
		}
		else
		{
			Auswahl.Count = Auswahl.MinEintraege;
			Auswahl.Changed = FALSE;
		}
	}
	
	// Zeit des letzten Schritts merken
	enc_time = Zeit.totSekunden;
}
