/**
* \file fan.h
* \brief In dieser Datei sind Funktionen zur regelung des Luefters sind
* \version 
* v. 1.1 Temperatren angepasst
* v. 1.0 Erste Version
* \date
* Create: 29.07.2008<br>
* Modify: 06.09.2008
* \author Philipp Kaelin 
*/

#ifndef FAN_H
	#define FAN_H

	//***********
	//* Includes
	//***********
	#include <avr/io.h>
	#include "global.h"


	//**********
	//* Defines
	//**********

	// Hardware
	#define FAN_DDR			DDRD								// PWM Ausgang fr Lfter
	#define FAN_PORT		PORTD
	#define FAN_PIN			PIND
	#define FAN_PINNR		PD5
	
	#define FAN_TIMER_PWM	OCR1A								// Vergleichsregister des PWM Timers

	// Einstellungen
	#define FAN_MINV		7000								// Minimalspannung in mV
	#define FAN_TON			350									// Einschalttemperatur in (C * 10)
	#define FAN_TOFF		300									// Ausschalttemperatur in (C * 10)
	#define FAN_TFULL		500									// Temperatur, bei der der Lfter mit voller geschwindigkeit luft in (C * 10)
	#define FAN_TEND		850									// Maximaltemperatur in (C * 10)

	
	#define FAN_MINPWM		(FAN_MINV / 12)						// Minimaler PWM
	#define FAN_DPWM		(1023 - FAN_MINPWM)					// Maximaler PWM Unterschied
	#define FAN_DTEMP		(FAN_TFULL - FAN_TON)				// Maximaler Temperaturunterschied


	//*************
	//* Funktionen
	//*************

	// Regelt den Lfter
	void ControlFan(void);

	
#endif
