/**
* \file ks0108.h
* \brief In dieser Datei sind Funktionen zur ansteuerung des LCDs
* \version 
* v 1.1 - Jede Steuerleitung kann jetzt einzeln definiert werden <br> - Die Daten koennen in ein High- und LowNibble aufgeteilt werden <br>
* v. 1.0 Erste Version von Fabian Maximilian Thiele
* \date
* Create: 09.02.2008<br>
* Modify: 09.08.2008
* \author  Philipp Kaelin (Orginal von Fabian Maximilian Thiele)
*/

#ifndef KS0108_H
	#define KS0108_H 

	//***********
	//* Includes
	//***********
	#include <avr/pgmspace.h>
	#include <stdlib.h>
	#include "ks0108_def.h"
	#include "global.h"
	#include "init.h"


	//************
	//* Variabeln
	//************
	typedef struct {
		unsigned char x;
		unsigned char y;
		unsigned char page;
	} lcdCoord;

	typedef unsigned char (*ks0108FontCallback)(const unsigned char*);


	//************************
	//* Function Declarations
	//************************
	
	// Hardware Functions
	void LCD_Dir_Out(void);
	void LCD_Dir_In(void);
	unsigned char LCD_Read_In(void);
	void LCD_Write_Out(unsigned char data);
	unsigned char LCD_Save_Cmd(void);
	void LCD_Restore_Cmd(unsigned char cmd);

	void ks0108Init(unsigned char invert);
	void ks0108Reset(unsigned char resetState);
	void ks0108Enable(void);
	unsigned char ks0108DoReadData(unsigned char first);
	unsigned char ks0108ReadData(void);
	void ks0108WriteCommand(unsigned char cmd, unsigned char chip);
	void ks0108WriteData(unsigned char data);

	// Graphic Functions
	void ks0108GotoXY(unsigned char x, unsigned char y);
	void ks0108FillRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char color);
	void ks0108InvertRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height);
	#define ks0108ClearScreen() {ks0108FillRect(0, 0, 127, 63, WHITE);}


	//**********
	//* Defines
	//**********
	// Hardware
	#define LCD_DI_LOW()		(LCD_DI_PORT &= ~(1 << LCD_DI_PINNR))
	#define LCD_RW_LOW()		(LCD_RW_PORT &= ~(1 << LCD_RW_PINNR))
	#define LCD_EN_LOW()		(LCD_EN_PORT &= ~(1 << LCD_EN_PINNR))
	#define LCD_CS1_LOW()		(LCD_CS1_PORT &= ~(1 << LCD_CS1_PINNR))
	#define LCD_CS2_LOW()		(LCD_CS2_PORT &= ~(1 << LCD_CS2_PINNR))
	#define LCD_RESET_LOW()		(LCD_RESET_PORT &= ~(1 << LCD_RESET_PINNR))

	#define LCD_DI_HIGH()		(LCD_DI_PORT |= (1 << LCD_DI_PINNR))
	#define LCD_RW_HIGH()		(LCD_RW_PORT |= (1 << LCD_RW_PINNR))
	#define LCD_EN_HIGH()		(LCD_EN_PORT |= (1 << LCD_EN_PINNR))
	#define LCD_CS1_HIGH()		(LCD_CS1_PORT |= (1 << LCD_CS1_PINNR))
	#define LCD_CS2_HIGH()		(LCD_CS2_PORT |= (1 << LCD_CS2_PINNR))
	#define LCD_RESET_HIGH()	(LCD_RESET_PORT |= (1 << LCD_RESET_PINNR))

	// Chips
	#define CHIP1				0x00
	#define CHIP2				0x01

	// Commands
	#define LCD_ON				0x3F
	#define LCD_OFF				0x3E
	#define LCD_SET_ADD			0x40
	#define LCD_SET_PAGE		0xB8
	#define LCD_DISP_START		0xC0

	// Colors
	#define BLACK				0xFF
	#define WHITE				0x00

	// Font Indices
	#define FONT_LENGTH			0
	#define FONT_FIXED_WIDTH	2
	#define FONT_HEIGHT			3
	#define FONT_FIRST_CHAR		4
	#define FONT_CHAR_COUNT		5
	#define FONT_WIDTH_TABLE	6

#endif
